//LED library
#include <FastLED.h>

//SD Card libraries
#include <SD.h> // need to include the SD library
#include <TMRpcm.h> //Arduino library for asynchronous playback of PCM/WAV files
#include <SPI.h> //  need to include the SPI library

//LED defines
#define LED_PIN     8
#define NUM_LEDS    7 
#define BRIGHTNESS  30
#define LED_TYPE    WS2811
#define COLOR_ORDER GRB
CRGB leds[NUM_LEDS];
#define UPDATES_PER_SECOND 100

int isPaused = 0;

CRGBPalette16 currentPalette;
TBlendType    currentBlending;

extern CRGBPalette16 myRedWhiteBluePalette;
extern const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM;

//SD card defines
#define SD_ChipSelectPin 4 //connect pin 4 of arduino to cs pin of sd card
TMRpcm tmrpcm; // create an object for use in this sketch
int temp = 1;
int pp = 5;
int next = 6;
int prev = 7;


void setup() {
  /*========== LED setup ==========*/
  delay( 3000 ); // power-up safety delay
  FastLED.addLeds<LED_TYPE, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection( TypicalLEDStrip );
  FastLED.setBrightness(  BRIGHTNESS );

  currentPalette = RainbowColors_p;
  currentBlending = LINEARBLEND;
  /*========== LED setup ==========*/

  /*========== SD card and speakers setup ==========*/
  pinMode(pp, INPUT_PULLUP);
  pinMode(next, INPUT_PULLUP);
  pinMode(prev, INPUT_PULLUP);

  tmrpcm.speakerPin = 9;
  Serial.begin(9600);
  if (!SD.begin(SD_ChipSelectPin)) // returns 1 if the card is present
  {
    Serial.println("SD fail");
    return;
  }

  tmrpcm.setVolume(5);
  char song[] = "1.wav";
  tmrpcm.play(song); //the sound file "song" will play each time the arduino powers up, or is reset
  /*========== SD card and speakers setup ==========*/


}


void loop()
{
  /*========== LEDs ==========*/
  if (isPaused % 2 == 0){
    ChangePalettePeriodically();
  }
  else
  {
    makeLedsOff();
  }
  static uint8_t startIndex = 0;
  startIndex = startIndex + 1; /* motion speed */

  FillLEDsFromPaletteColors( startIndex);

  FastLED.show();
  //FastLED.delay(1000 / UPDATES_PER_SECOND);
  /*========== LEDs ==========*/

  /*========== Play the songs from de micro SD card ==========*/
  while (digitalRead(pp) == 0 || digitalRead(next) == 0 || digitalRead(prev) == 0)
  {
    if (digitalRead(pp) == 0)
    {
      tmrpcm.pause();
      isPaused++;
      while (digitalRead(pp) == 0);
      //delay(200);
    }
    else if (digitalRead(next) == 0)
    {
      if (temp < 6) //temp should be lesser than no. of songs
        temp = temp + 1;
      if(temp == 6)
        temp = 1;
      while (digitalRead(next) == 0);
      //delay(200);
      FastLED.delay(1000 / UPDATES_PER_SECOND);
      song();
    }
    else if (digitalRead(prev) == 0)
    {
      if (temp > 1)
        temp = temp - 1;
      while (digitalRead(prev) == 0);
      //delay(200);
      FastLED.delay(1000 / UPDATES_PER_SECOND);
      song();
    }
  }
  /*========== Play the songs from de micro SD card ==========*/

}

void song (void)
{
  if (temp == 1)
  {
    char song[] = "1.wav";
    tmrpcm.play(song);
  }
  else if (temp == 2)
  {
    char song[] = "2.wav";
    tmrpcm.play(song);
  }
  else if (temp == 3)
  {
    char song[] = "3.wav";
    tmrpcm.play(song);
  }
  else if (temp == 4)
  {
    char song[] = "4.wav";
    tmrpcm.play(song);
  }
  else if (temp == 5)
  {
    char song[] = "5.wav";
    tmrpcm.play(song);
  }
}

void makeLedsOff()
{
  for ( int i = 0; i < 16; i++) {
    currentPalette[i] = CRGB::Black;
  }
}

void FillLEDsFromPaletteColors( uint8_t colorIndex)
{
  uint8_t brightness = 255;

  for ( int i = 0; i < NUM_LEDS; i++) {
    leds[i] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
    colorIndex += 3;
  }
}

void ChangePalettePeriodically()
{
  uint8_t secondHand = (millis() / 1000) % 60;
  static uint8_t lastSecond = 99;

  if ( lastSecond != secondHand) {
    lastSecond = secondHand;
    if ( secondHand ==  0)  {
      currentPalette = RainbowColors_p;
      currentBlending = LINEARBLEND;
    }
    if ( secondHand == 10)  {
      currentPalette = RainbowStripeColors_p;
      currentBlending = NOBLEND;
    }
    if ( secondHand == 15)  {
      currentPalette = RainbowStripeColors_p;
      currentBlending = LINEARBLEND;
    }
    if ( secondHand == 20)  {
      SetupPurpleAndGreenPalette();
      currentBlending = LINEARBLEND;
    }
    if ( secondHand == 25)  {
      SetupTotallyRandomPalette();
      currentBlending = LINEARBLEND;
    }
    if ( secondHand == 30)  {
      SetupBlackAndWhiteStripedPalette();
      currentBlending = NOBLEND;
    }
    if ( secondHand == 35)  {
      SetupBlackAndWhiteStripedPalette();
      currentBlending = LINEARBLEND;
    }
    if ( secondHand == 40)  {
      currentPalette = CloudColors_p;
      currentBlending = LINEARBLEND;
    }
    if ( secondHand == 45)  {
      currentPalette = PartyColors_p;
      currentBlending = LINEARBLEND;
    }
    if ( secondHand == 50)  {
      currentPalette = myRedWhiteBluePalette_p;
      currentBlending = NOBLEND;
    }
    if ( secondHand == 55)  {
      currentPalette = myRedWhiteBluePalette_p;
      currentBlending = LINEARBLEND;
    }
  }
}

// This function fills the palette with totally random colors.
void SetupTotallyRandomPalette()
{
  for ( int i = 0; i < 16; i++) {
    currentPalette[i] = CHSV( random8(), 255, random8());
  }
}

// This function sets up a palette of black and white stripes,
// using code.
void SetupBlackAndWhiteStripedPalette()
{
  // 'black out' all 16 palette entries...
  fill_solid( currentPalette, 16, CRGB::Black);
  // and set every fourth one to white.
  currentPalette[0] = CRGB::Pink;
  currentPalette[4] = CRGB::PaleTurquoise;
  currentPalette[8] = CRGB::Orange;
  currentPalette[12] = CRGB::Green;

}

// This function sets up a palette of purple and green stripes.
void SetupPurpleAndGreenPalette()
{
  CRGB purple = CHSV( HUE_PURPLE, 255, 255);
  CRGB green  = CHSV( HUE_GREEN, 255, 255);
  CRGB black  = CRGB::Black;

  currentPalette = CRGBPalette16(
                     green,  green,  black,  black,
                     purple, purple, black,  black,
                     green,  green,  black,  black,
                     purple, purple, black,  black );
}

const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM =
{
  CRGB::Red,
  CRGB::Gray, // 'white' is too bright compared to red and blue
  CRGB::Blue,
  CRGB::Black,

  CRGB::Red,
  CRGB::Gray,
  CRGB::Blue,
  CRGB::Black,

  CRGB::Red,
  CRGB::Red,
  CRGB::Gray,
  CRGB::Gray,
  CRGB::Blue,
  CRGB::Blue,
  CRGB::Black,
  CRGB::Black
};
